'''
Created on 11/07/2016

@author: MMPE
'''
import unittest
import os

tfp = os.path.join(os.path.dirname(__file__), 'test_files/')
class Test(unittest.TestCase):
    pass
        


if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testUnix_filename']
    unittest.main()
