@echo off  
echo.
ECHO Syntax: 
echo ------------------------------------------------
ECHO ascii2bin.exe {ascii_file.sel} [{bin_file}]
echo ------------------------------------------------
echo.
echo if {bin_file} is omitted "test.sel" is converted into "xxx_bin.sel"
echo.
echo.
cmd.exe
echo ascii2bin.exe
